VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Features"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Dim CallController As CallController

Private Enum FeatureStates
    fsWelcome = 0
    fsForward = 1
    fsHold = 2
    fsRecord = 3
    fsTransfer = 4
    fsPlayCallerID = 5
    fsHangUp = 6
    fsError = 99
End Enum

Private Enum RecordStates
    rsPlayExplanation = 0
    rsRecordAudio = 1
    rsPlaybackAudio = 2
    rsEndState = 3
End Enum
    
Dim RecordState As RecordStates
Dim FeatureState As FeatureStates
Dim CurrentDigit As String

Implements ITVCallHandler

Private Sub Class_Terminate()
    Set CallController = Nothing
End Sub

Private Sub ITVCallHandler_Init(objCallController As TERAVoiceLib.CallController)
    Set CallController = objCallController
End Sub

Private Sub ITVCallHandler_DigitReceived(Digit As String)


    'In this sample we will dispatch all processing through the
    'ITVCallHandler_TimePulse function. This function always gets
    'called immediately after ITVCallHandler_DigitReceived, so
    'we simply store the current received digit only
    
    CurrentDigit = Digit
    
        
End Sub

Private Sub ITVCallHandler_FileSaved(FileName As String, Length As Long)
    'This procedure is not needed in this sample
    'All procedures of ITVCallHandler must be implemented though!
End Sub

Private Sub ITVCallHandler_PlayDone()

End Sub

Private Sub ITVCallHandler_TimePulse()
Dim Result As String
Dim i As Long
Dim Digit As String

    'Ensure that we process each digit only once by clearing
    'the digit saved in the ITVCallHandler_DigitReceived function
    Digit = CurrentDigit
    CurrentDigit = ""
    
    Select Case FeatureState
    Case fsWelcome
        Select Case Digit
        Case "1"
            FeatureState = fsForward
            CallController.StopPlayRecord
            Exit Sub
        Case "2"
            FeatureState = fsHold
            CallController.StopPlayRecord
            Exit Sub
        Case "3"
            FeatureState = fsRecord
            CallController.StopPlayRecord
            Exit Sub
        Case "4"
            FeatureState = fsTransfer
            CallController.StopPlayRecord
            Exit Sub
        Case "5"
            FeatureState = fsPlayCallerID
            CallController.StopPlayRecord
            Exit Sub
        Case "6"
            FeatureState = fsHangUp
            CallController.HangUp
            Exit Sub
        Case Else
            'Ignore
        End Select
        If Not CallController.IsPlaying Then
            If Not CallController.PlayAudioFile(App.Path & "\Welcome.wav") Then
                CallController.WriteLogFile 12, "Unable to play audio file", True
                CallController.HangUp
            End If
        End If
    Case fsForward
        HandleForward Digit
    Case fsHold
        HandleHold
    Case fsRecord
        HandleRecord
    Case fsTransfer
        HandleTransfer Digit
    Case fsPlayCallerID
        HandlePlayCallerID
    Case fsError
        If Not CallController.IsPlaying Then
            If Not CallController.PlayAudioFile(App.Path & "\Error.wav") Then
                CallController.WriteLogFile 14, "Unable to play audio file", True
                CallController.HangUp
            End If
        End If
    End Select
        

    
End Sub

Private Sub HandleRecord()

    Select Case RecordState
    Case rsPlayExplanation
        If Not CallController.PlayAudioFile(App.Path & "\ExplainRecord.wav") Then
            CallController.WriteLogFile 40, "Unable to play audio file", True
            CallController.HangUp
        End If
        RecordState = rsRecordAudio
    Case rsRecordAudio
        If CallController.IsPlaying = False And CallController.IsRecording = False Then
            If Not CallController.RecordAudioFile(App.Path & "\Record.wav", True, 10) Then
                CallController.WriteLogFile 42, "Unable to record audio", True
                CallController.HangUp
            End If
            RecordState = rsPlaybackAudio
        End If
    Case rsPlaybackAudio
        If CallController.IsPlaying = False And CallController.IsRecording = False Then
            If Not CallController.PlayAudioFile(App.Path & "\Record.wav") Then
                CallController.WriteLogFile 44, "Unable to play audio file", True
                CallController.HangUp
            End If
            RecordState = rsEndState
        End If
    Case rsEndState
        If CallController.IsPlaying = False And CallController.IsRecording = False Then
            RecordState = rsPlayExplanation
            FeatureState = fsWelcome
        End If
    End Select
    
    
    
End Sub

Private Sub HandleHold()
Static HoldUntil As Date

    If CallController.IsOnHold Then
        If Now >= HoldUntil Then
            CallController.UnHoldCall
            FeatureState = fsWelcome
        End If
    Else
        HoldUntil = DateAdd("s", 10, Now)
        If Not CallController.HoldCall Then
            CallController.WriteLogFile 30, "Unable to put call on hold", True
            FeatureState = fsWelcome
        End If
    End If

End Sub



Private Sub HandleForward(Digit As String)
Static TargetID As String
Dim lngTargetID As Long

    Select Case Digit
    Case "0", "1", "2", "3", "4", "5", "6", "7", "8", "9"
        TargetID = TargetID & Digit
        CallController.StopPlayRecord
        Exit Sub
    Case "#"
        If TargetID = "" Then
            FeatureState = fsWelcome
            Exit Sub
        End If
        lngTargetID = Val(TargetID)
        TargetID = ""
        If Not CallController.GotoCallHandler(lngTargetID) Then
            If Not CallController.PlayAudioFile(App.Path & "\UnableToForward.wav") Then
                CallController.WriteLogFile 22, "Unable to play audio file", True
                CallController.HangUp
            End If
            FeatureState = fsWelcome
            Exit Sub
        End If
    Case Else
        If (Not CallController.IsPlaying) And (TargetID = "") Then
            If Not CallController.PlayAudioFile(App.Path & "\ExplainForward.wav") Then
                CallController.WriteLogFile 20, "Unable to play audio file", True
                CallController.HangUp
            End If
        End If
    End Select
    



End Sub

Private Sub ITVCallHandler_UnInit()

    'Important: Do not forget to destroy this object!!!
    Set CallController = Nothing
    
    'Also do not forget to destroy object references,
    'otherwise this object might never get unloaded
    
End Sub

Private Sub HandleTransfer(Digit As String)
Static TargetID As String
Dim TargetNumber As String

    Select Case Digit
    Case "0", "1", "2", "3", "4", "5", "6", "7", "8", "9"
        TargetID = TargetID & Digit
        CallController.StopPlayRecord
        Exit Sub
    Case "#"
        If TargetID = "" Then
            FeatureState = fsWelcome
            Exit Sub
        End If
        TargetNumber = TargetID
        TargetID = ""
        If Not CallController.TransferCall(TargetNumber, 0) Then
            If Not CallController.PlayAudioFile(App.Path & "\UnableToTransfer.wav") Then
                CallController.WriteLogFile 50, "Unable to play audio file", True
                CallController.HangUp
            End If
            FeatureState = fsWelcome
            Exit Sub
        End If
    Case Else
        If (Not CallController.IsPlaying) And (TargetID = "") Then
            If Not CallController.PlayAudioFile(App.Path & "\ExplainTransfer.wav") Then
                CallController.WriteLogFile 52, "Unable to play audio file", True
                CallController.HangUp
            End If
        End If
    End Select
    



End Sub

Private Sub HandlePlayCallerID()
Dim Files() As String
Dim Suffix As String
Dim n As Long


    ReDim Files(Len(CallController.CallerID))
    If Len(CallController.CallerID) = 0 Then
        Files(0) = CallController.SysAudioFolder & "\CallerIDunknown.wav"
    Else
        Files(0) = CallController.SysAudioFolder & "\CallerIDis.wav"
    End If
    Suffix = "A"
    For n = 1 To UBound(Files)
        If n = UBound(Files) - 1 Then Suffix = "B"
        If n = UBound(Files) Then Suffix = "A"
        Files(n) = CallController.SysAudioFolder & "\" & Mid(CallController.CallerID, n, 1) & Suffix & ".wav"
        If Suffix = "A" Then
            Suffix = "B"
        Else
            Suffix = "A"
        End If
    Next
    
    If Not CallController.PlayAudioFileList(Files()) Then
        CallController.WriteLogFile 60, "Unable to play CallerID", True
        CallController.HangUp
    End If
    
    FeatureState = fsWelcome
    

End Sub
